function [x err Es] = active_set(A,b,c,K,iseq,varargin)
% function [x err Es] = active_set(A,b,c,K,iseq,varargin)
% 
% Solve a mixed LP/SDP.  iseq is a vector of booleans, true for
% equality constraints in (A,b) and false for the inequality
% constraints.  Inequality constraints are added iteratively for
% efficiency and maintained using slacks.


frac = 0.05;
limit = 1.00;
errtol = 1e-2;
fid = 1;
init = 0.00;
addstrat = 'frac';

i=1;
while i<=length(varargin)
  if isstr(varargin{i})
    switch varargin{i}
      case 'frac', frac=varargin{i+1};i=i+1;     % fraction of violated constraints to add
      case 'asfid', fid=varargin{i+1};i=i+1;
      case 'limit', limit=varargin{i+1};i=i+1;
      case 'addstrat', addstrat=varargin{i+1};i=i+1;
      case 'errtol', errtol=varargin{i+1};i=i+1;
    end
  end
  i=i+1;
end

[m n]=size(A);
if ~isfield(K,'l'), K.l = 0; end

Aeq = A( iseq,:);
beq = b( iseq,:);
neq = length(beq);
Ain = A(~iseq,:);
bin = b(~iseq,:);
nin = length(bin);

x = sparse(n,1);
inmask = logical(zeros(nin,1));
inidx = [];
nnew = round(frac*nin);

bin_norm = sum(abs(bin));
if nin ==0 
  init = limit;
  bin_norm = 1;
end


fprintf('%d equality  %d inequality  %d linear  %d sdp\n',...
        sum(iseq), sum(~iseq), K.l, K.s);

fprintf('%2s %6s %4s %8s %8s %6s %7s\n',...
        'ii', 'nineq', 'frac', 'inerr','e4to3','cviter','minutes');

astime = 0;
ii=0;

f = 0.0;
while f < limit
  ii = ii+1;
  if ii==1
    newidx = [];
  else 
    availidx = find(~inmask);
    inres = Ain(availidx,:) * x - bin(availidx);
    switch addstrat
      case 'frac'
        [tmp ord] = sort(inres,'ascend');
        toadd = min( nnew, length(ord));
        newidx = availidx(ord(1:toadd));
      case 'adapt'
        newidx = availidx(inres < 0);
    end
  end
  f = f + length(newidx)/nin;
  f = min(limit,f);

  inmask(newidx) = 1;
  
  nslack = sum(inmask);
  Ks = K;
  Ks.l = Ks.l + nslack;

  As = [  sp0(neq,nslack)     Aeq
         -spI(nslack,nslack)  Ain(inmask,:)];
  bs = [ beq; bin(inmask) ];
  cs = [ sp0(nslack,1); c];
  
  Es = convex_iter(As, bs, cs, Ks, varargin{:});
  astime = astime + Es.totaltime;
  x = Es.xh(nslack+1:length(Es.xh));
  inres = Ain*x - bin;
  err = abs(sum(inres(inres<0))) / bin_norm;

  fprintf(fid,'%2d %6d %4.2f %8.2e %8.2e %6d %7.2f\n',...
          ii, nslack, f, err, Es.e4to3, Es.last, Es.totaltime/60);
  
  if err < errtol
    break;
  end

end

Es.totaltime = astime;